@@ -22,13 +22,13 @@
 
 #include "qwebplugindatabase_p.h"
 
-#include "AbstractDatabase.h"
 #include "ApplicationCacheStorage.h"
 #include "CrossOriginPreflightResultCache.h"
-#include "DatabaseTracker.h"
+#include "DatabaseManager.h"
 #include "FileSystem.h"
 #include "FontCache.h"
 #include "GCController.h"
+#include "GroupSettings.h"
 #include "IconDatabase.h"
 #include "Image.h"
 #if ENABLE(ICONDATABASE)
@@ -44,6 +44,8 @@
 #include "PluginDatabase.h"
 #include "RuntimeEnabledFeatures.h"
 #include "Settings.h"
+#include "StorageThread.h"
+#include "WorkerThread.h"
 #include <QDir>
 #include <QFileInfo>
 #include <QFont>
@@ -52,11 +54,12 @@
 #include <QSharedData>
 #include <QStandardPaths>
 #include <QUrl>
+#include <wtf/FastMalloc.h>
 #include <wtf/text/WTFString.h>
 
 
 
-void QWEBKIT_EXPORT qt_networkAccessAllowed(bool isAllowed)
+QWEBKIT_EXPORT void qt_networkAccessAllowed(bool isAllowed)
 {
 #ifndef QT_NO_BEARERMANAGEMENT
     WebCore::networkStateNotifier().setNetworkAccessAllowed(isAllowed);
@@ -65,8 +68,10 @@ void QWEBKIT_EXPORT qt_networkAccessAllowed(bool isAllowed)
 
 class QWebSettingsPrivate {
 public:
-    QWebSettingsPrivate(WebCore::Settings* wcSettings = 0)
-        : settings(wcSettings)
+    QWebSettingsPrivate(WebCore::Settings* wcSettings = 0, WebCore::GroupSettings* wcGroupSettings = 0)
+        : offlineStorageDefaultQuota(0)
+        , settings(wcSettings)
+        , groupSettings(wcGroupSettings)
     {
     }
 
@@ -77,11 +82,13 @@ public:
     QString defaultTextEncoding;
     QString localStoragePath;
     QString offlineWebApplicationCachePath;
+    QString offlineDatabasePath;
     QString mediaType;
     qint64 offlineStorageDefaultQuota;
     QWebSettings::ThirdPartyCookiePolicy thirdPartyCookiePolicy;
     void apply();
     WebCore::Settings* settings;
+    WebCore::GroupSettings* groupSettings;
 };
 
 Q_GLOBAL_STATIC(QList<QWebSettingsPrivate*>, allSettings);
@@ -142,17 +149,22 @@ void QWebSettingsPrivate::apply()
         settings->setDNSPrefetchingEnabled(value);
 
         value = attributes.value(QWebSettings::JavascriptEnabled,
-                                      global->attributes.value(QWebSettings::JavascriptEnabled));
+                                 global->attributes.value(QWebSettings::JavascriptEnabled));
         settings->setScriptEnabled(value);
 #if USE(ACCELERATED_COMPOSITING)
         value = attributes.value(QWebSettings::AcceleratedCompositingEnabled,
-                                      global->attributes.value(QWebSettings::AcceleratedCompositingEnabled));
-
+                                 global->attributes.value(QWebSettings::AcceleratedCompositingEnabled));
         settings->setAcceleratedCompositingEnabled(value);
-        settings->setAcceleratedCompositingFor3DTransformsEnabled(value);
-        settings->setAcceleratedCompositingForAnimationEnabled(value);
-        settings->setAcceleratedCompositingForVideoEnabled(false);
-        settings->setAcceleratedCompositingForPluginsEnabled(false);
+
+#if ENABLE(ACCELERATED_2D_CANVAS)
+        value = value && attributes.value(QWebSettings::Accelerated2dCanvasEnabled,
+                                          global->attributes.value(QWebSettings::Accelerated2dCanvasEnabled));
+        settings->setAccelerated2dCanvasEnabled(value);
+#endif
+
+        bool showDebugVisuals = qgetenv("WEBKIT_SHOW_COMPOSITING_DEBUG_VISUALS") == "1";
+        settings->setShowDebugBorders(showDebugVisuals);
+        settings->setShowRepaintCounter(showDebugVisuals);
 #endif
 #if ENABLE(WEBGL)
         value = attributes.value(QWebSettings::WebGLEnabled,
@@ -163,14 +175,16 @@ void QWebSettingsPrivate::apply()
         // For now, enable CSS shaders when WebGL is enabled.
         settings->setCSSCustomFilterEnabled(value);
 #endif
-#if USE(ACCELERATED_COMPOSITING)
-        settings->setAcceleratedCompositingForCanvasEnabled(value);
 #endif
+#if ENABLE(WEB_AUDIO)
+        value = attributes.value(QWebSettings::WebAudioEnabled, global->attributes.value(QWebSettings::WebAudioEnabled));
+        settings->setWebAudioEnabled(value);
 #endif
 
         value = attributes.value(QWebSettings::CSSRegionsEnabled,
                                  global->attributes.value(QWebSettings::CSSRegionsEnabled));
         WebCore::RuntimeEnabledFeatures::setCSSRegionsEnabled(value);
+
         value = attributes.value(QWebSettings::CSSGridLayoutEnabled,
                                  global->attributes.value(QWebSettings::CSSGridLayoutEnabled));
         settings->setCSSGridLayoutEnabled(value);
@@ -233,10 +247,21 @@ void QWebSettingsPrivate::apply()
                                       global->attributes.value(QWebSettings::PrintElementBackgrounds));
         settings->setShouldPrintBackgrounds(value);
 
-#if ENABLE(SQL_DATABASE)
         value = attributes.value(QWebSettings::OfflineStorageDatabaseEnabled,
                                       global->attributes.value(QWebSettings::OfflineStorageDatabaseEnabled));
-        WebCore::AbstractDatabase::setIsAvailable(value);
+#if ENABLE(SQL_DATABASE)
+        WebCore::DatabaseManager::manager().setIsAvailable(value);
+#endif
+
+#if ENABLE(INDEXED_DATABASE)
+        QString path = !offlineDatabasePath.isEmpty() ? offlineDatabasePath : global->offlineDatabasePath;
+        Q_ASSERT(groupSettings);
+        // Setting the path to empty string disables persistent storage of the indexed database.
+        if (!value)
+            path = QString();
+        groupSettings->setIndexedDBDatabasePath(path);
+        qint64 quota = offlineStorageDefaultQuota ? offlineStorageDefaultQuota : global->offlineStorageDefaultQuota;
+        groupSettings->setIndexedDBQuotaBytes(quota);
 #endif
 
         value = attributes.value(QWebSettings::OfflineWebApplicationCacheEnabled,
@@ -247,9 +272,10 @@ void QWebSettingsPrivate::apply()
                                       global->attributes.value(QWebSettings::LocalStorageEnabled));
         settings->setLocalStorageEnabled(value);
 
-        value = attributes.value(QWebSettings::LocalContentCanAccessRemoteUrls,
+        bool remoteAccess = attributes.value(QWebSettings::LocalContentCanAccessRemoteUrls,
                                       global->attributes.value(QWebSettings::LocalContentCanAccessRemoteUrls));
-        settings->setAllowUniversalAccessFromFileURLs(value);
+        settings->setAllowUniversalAccessFromFileURLs(remoteAccess);
+        settings->setAllowRemoteAccessFromFileURLs(remoteAccess);
 
         value = attributes.value(QWebSettings::LocalContentCanAccessFileUrls,
                                       global->attributes.value(QWebSettings::LocalContentCanAccessFileUrls));
@@ -268,7 +294,7 @@ void QWebSettingsPrivate::apply()
 #if ENABLE(SMOOTH_SCROLLING)
         value = attributes.value(QWebSettings::ScrollAnimatorEnabled,
                                       global->attributes.value(QWebSettings::ScrollAnimatorEnabled));
-        settings->setEnableScrollAnimator(value);
+        settings->setScrollAnimatorEnabled(value);
 #endif
 
         value = attributes.value(QWebSettings::CaretBrowsingEnabled,
@@ -351,7 +377,7 @@ QWebSettings* QWebSettings::globalSettings()
     setOfflineStoragePath() with an appropriate file path, and can limit the quota
     for each application by calling setOfflineStorageDefaultQuota().
 
-    \sa QWebPage::settings(), QWebView::settings(), {Web Browser}
+    \sa QWebPage::settings(), QWebView::settings(), {Tab Browser}
 */
 
 /*!
@@ -491,10 +517,20 @@ QWebSettings* QWebSettings::globalSettings()
         This is disabled by default.
     \value SiteSpecificQuirksEnabled This setting enables WebKit's workaround for broken sites. It is
         enabled by default.
+    \value CSSGridLayoutEnabled This setting enables support for the CSS 3 Grid Layout module. This
+        CSS module is currently only a draft and support for it is disabled by default.
+    \value CSSRegionsEnabled This setting enables support for the CSS 3 Regions module. This
+        CSS module is currently only a draft and support for it is enabled by default.
     \value ScrollAnimatorEnabled This setting enables animated scrolling. It is disabled by default.
     \value CaretBrowsingEnabled This setting enables caret browsing. It is disabled by default.
     \value NotificationsEnabled Specifies whether support for the HTML 5 web notifications is enabled
         or not. This is enabled by default.
+    \value Accelerated2dCanvasEnabled Specifies whether the HTML5 2D canvas should be a OpenGL framebuffer.
+        This makes many painting operations faster, but slows down pixel access. This is disabled by default.
+    \value WebGLEnabled This setting enables support for WebGL.
+        It is enabled by default.
+    \value HyperlinkAuditingEnabled This setting enables support for hyperlink auditing (<a ping>).
+        It is disabled by default.
 */
 
 /*!
@@ -540,6 +576,7 @@ QWebSettings::QWebSettings()
     d->attributes.insert(QWebSettings::LocalContentCanAccessFileUrls, true);
     d->attributes.insert(QWebSettings::AcceleratedCompositingEnabled, true);
     d->attributes.insert(QWebSettings::WebGLEnabled, true);
+    d->attributes.insert(QWebSettings::WebAudioEnabled, false);
     d->attributes.insert(QWebSettings::CSSRegionsEnabled, true);
     d->attributes.insert(QWebSettings::CSSGridLayoutEnabled, false);
     d->attributes.insert(QWebSettings::HyperlinkAuditingEnabled, false);
@@ -549,6 +586,7 @@ QWebSettings::QWebSettings()
     d->attributes.insert(QWebSettings::ScrollAnimatorEnabled, false);
     d->attributes.insert(QWebSettings::CaretBrowsingEnabled, false);
     d->attributes.insert(QWebSettings::NotificationsEnabled, true);
+    d->attributes.insert(QWebSettings::Accelerated2dCanvasEnabled, false);
     d->offlineStorageDefaultQuota = 5 * 1024 * 1024;
     d->defaultTextEncoding = QLatin1String("iso-8859-1");
     d->thirdPartyCookiePolicy = AlwaysAllowThirdPartyCookies;
@@ -557,10 +595,9 @@ QWebSettings::QWebSettings()
 /*!
     \internal
 */
-QWebSettings::QWebSettings(WebCore::Settings* settings)
-    : d(new QWebSettingsPrivate(settings))
+QWebSettings::QWebSettings(WebCore::Settings* settings, WebCore::GroupSettings* groupSettings)
+    : d(new QWebSettingsPrivate(settings, groupSettings))
 {
-    d->settings = settings;
     d->apply();
     allSettings()->append(d);
 }
@@ -829,7 +866,6 @@ void QWebSettings::clearMemoryCaches()
     int pageCapacity = WebCore::pageCache()->capacity();
     // Setting size to 0, makes all pages be released.
     WebCore::pageCache()->setCapacity(0);
-    WebCore::pageCache()->releaseAutoreleasedPagesNow();
     WebCore::pageCache()->setCapacity(pageCapacity);
 
     // Invalidating the font cache and freeing all inactive font data.
@@ -842,6 +878,13 @@ void QWebSettings::clearMemoryCaches()
     WebCore::gcController().discardAllCompiledCode();
     // Garbage Collect to release the references of CachedResource from dead objects.
     WebCore::gcController().garbageCollectNow();
+
+    // FastMalloc has lock-free thread specific caches that can only be cleared from the thread itself.
+    WebCore::StorageThread::releaseFastMallocFreeMemoryInAllThreads();
+#if ENABLE(WORKERS)
+    WebCore::WorkerThread::releaseFastMallocFreeMemoryInAllThreads();
+#endif
+    WTF::releaseFastMallocFreeMemory();        
 }
 
 /*!
@@ -895,6 +938,7 @@ void QWebSettings::setObjectCacheCapacities(int cacheMinDeadCapacity, int cacheM
     WebCore::memoryCache()->setCapacities(qMax(0, cacheMinDeadCapacity),
                                     qMax(0, cacheMaxDead),
                                     qMax(0, totalCapacity));
+    WebCore::memoryCache()->setDeadDecodedDataDeletionInterval(disableCache ? 0 : 60);
 }
 
 /*!
@@ -1036,8 +1080,9 @@ void QWebSettings::resetAttribute(WebAttribute attr)
 void QWebSettings::setOfflineStoragePath(const QString& path)
 {
     WebCore::initializeWebCoreQt();
+    QWebSettings::globalSettings()->d->offlineDatabasePath = path;
 #if ENABLE(SQL_DATABASE)
-    WebCore::DatabaseTracker::tracker().setDatabaseDirectoryPath(path);
+    WebCore::DatabaseManager::manager().setDatabaseDirectoryPath(path);
 #endif
 }
 
@@ -1052,11 +1097,7 @@ void QWebSettings::setOfflineStoragePath(const QString& path)
 QString QWebSettings::offlineStoragePath()
 {
     WebCore::initializeWebCoreQt();
-#if ENABLE(SQL_DATABASE)
-    return WebCore::DatabaseTracker::tracker().databaseDirectoryPath();
-#else
-    return QString();
-#endif
+    return QWebSettings::globalSettings()->d->offlineDatabasePath;
 }
 
 /*!
@@ -1170,7 +1211,7 @@ void QWebSettings::setLocalStoragePath(const QString& path)
     \since 4.6
 
     Returns the path for HTML5 local storage.
-    
+
     \sa setLocalStoragePath()
 */
 QString QWebSettings::localStoragePath() const
