@@ -38,7 +38,6 @@
 #include <CoreFoundation/CoreFoundation.h>
 #include <WebCore/BString.h>
 #include <WebCore/HistoryItem.h>
-#include <WebCore/HistoryPropertyList.h>
 #include <WebCore/KURL.h>
 #include <WebCore/PageGroup.h>
 #include <WebCore/SharedBuffer.h>
@@ -49,46 +48,6 @@
 using namespace WebCore;
 using namespace std;
 
-CFStringRef DatesArrayKey = CFSTR("WebHistoryDates");
-CFStringRef FileVersionKey = CFSTR("WebHistoryFileVersion");
-
-#define currentFileVersion 1
-
-class WebHistoryWriter : public HistoryPropertyListWriter {
-public:
-    WebHistoryWriter(const WebHistory::DateToEntriesMap&);
-
-private:
-    virtual void writeHistoryItems(BinaryPropertyListObjectStream&);
-
-    const WebHistory::DateToEntriesMap& m_entriesByDate;
-    Vector<WebHistory::DateKey> m_dateKeys;
-};
-
-WebHistoryWriter::WebHistoryWriter(const WebHistory::DateToEntriesMap& entriesByDate)
-    : m_entriesByDate(entriesByDate)
-{
-    copyKeysToVector(m_entriesByDate, m_dateKeys);
-    sort(m_dateKeys.begin(), m_dateKeys.end());
-}
-
-void WebHistoryWriter::writeHistoryItems(BinaryPropertyListObjectStream& stream)
-{
-    for (int dateIndex = m_dateKeys.size() - 1; dateIndex >= 0; --dateIndex) {
-        // get the entries for that date
-        CFArrayRef entries = m_entriesByDate.get(m_dateKeys[dateIndex]).get();
-        CFIndex entriesCount = CFArrayGetCount(entries);
-        for (CFIndex j = entriesCount - 1; j >= 0; --j) {
-            IWebHistoryItem* item = (IWebHistoryItem*) CFArrayGetValueAtIndex(entries, j);
-            COMPtr<WebHistoryItem> webItem(Query, item);
-            if (!webItem)
-                continue;
-
-            writeHistoryItem(stream, webItem->historyItem());
-        }
-    }
-}
-
 static bool areEqualOrClose(double d1, double d2)
 {
     double diff = d1-d2;
@@ -97,10 +56,10 @@ static bool areEqualOrClose(double d1, double d2)
 
 static COMPtr<CFDictionaryPropertyBag> createUserInfoFromArray(BSTR notificationStr, CFArrayRef arrayItem)
 {
-    RetainPtr<CFMutableDictionaryRef> dictionary(AdoptCF, 
+    RetainPtr<CFMutableDictionaryRef> dictionary = adoptCF(
         CFDictionaryCreateMutable(0, 0, &kCFTypeDictionaryKeyCallBacks, &kCFTypeDictionaryValueCallBacks));
 
-    RetainPtr<CFStringRef> key(AdoptCF, MarshallingHelpers::BSTRToCFStringRef(notificationStr));
+    RetainPtr<CFStringRef> key = adoptCF(MarshallingHelpers::BSTRToCFStringRef(notificationStr));
     CFDictionaryAddValue(dictionary.get(), key.get(), arrayItem);
 
     COMPtr<CFDictionaryPropertyBag> result = CFDictionaryPropertyBag::createInstance();
@@ -111,7 +70,7 @@ static COMPtr<CFDictionaryPropertyBag> createUserInfoFromArray(BSTR notification
 static COMPtr<CFDictionaryPropertyBag> createUserInfoFromHistoryItem(BSTR notificationStr, IWebHistoryItem* item)
 {
     // reference counting of item added to the array is managed by the CFArray value callbacks
-    RetainPtr<CFArrayRef> itemList(AdoptCF, CFArrayCreate(0, (const void**) &item, 1, &MarshallingHelpers::kIUnknownArrayCallBacks));
+    RetainPtr<CFArrayRef> itemList = adoptCF(CFArrayCreate(0, (const void**) &item, 1, &MarshallingHelpers::kIUnknownArrayCallBacks));
     COMPtr<CFDictionaryPropertyBag> info = createUserInfoFromArray(notificationStr, itemList.get());
     return info;
 }
@@ -125,7 +84,7 @@ WebHistory::WebHistory()
     gClassCount++;
     gClassNameCount.add("WebHistory");
 
-    m_entriesByURL.adoptCF(CFDictionaryCreateMutable(0, 0, &kCFTypeDictionaryKeyCallBacks, &MarshallingHelpers::kIUnknownDictionaryValueCallBacks));
+    m_entriesByURL = adoptCF(CFDictionaryCreateMutable(0, 0, &kCFTypeDictionaryKeyCallBacks, &MarshallingHelpers::kIUnknownDictionaryValueCallBacks));
 
     m_preferences = WebPreferences::sharedStandardPreferences();
 }
@@ -234,176 +193,16 @@ HRESULT STDMETHODCALLTYPE WebHistory::setOptionalSharedHistory(
     return S_OK;
 }
 
-HRESULT STDMETHODCALLTYPE WebHistory::loadFromURL( 
-    /* [in] */ BSTR url,
-    /* [out] */ IWebError** error,
-    /* [retval][out] */ BOOL* succeeded)
+HRESULT STDMETHODCALLTYPE WebHistory::unused1()
 {
-    HRESULT hr = S_OK;
-    RetainPtr<CFMutableArrayRef> discardedItems(AdoptCF, 
-        CFArrayCreateMutable(0, 0, &MarshallingHelpers::kIUnknownArrayCallBacks));
-
-    RetainPtr<CFURLRef> urlRef(AdoptCF, MarshallingHelpers::BSTRToCFURLRef(url));
-
-    hr = loadHistoryGutsFromURL(urlRef.get(), discardedItems.get(), error);
-    if (FAILED(hr))
-        goto exit;
-
-    hr = postNotification(kWebHistoryLoadedNotification);
-    if (FAILED(hr))
-        goto exit;
-
-    if (CFArrayGetCount(discardedItems.get()) > 0) {
-        COMPtr<CFDictionaryPropertyBag> userInfo = createUserInfoFromArray(getNotificationString(kWebHistoryItemsDiscardedWhileLoadingNotification), discardedItems.get());
-        hr = postNotification(kWebHistoryItemsDiscardedWhileLoadingNotification, userInfo.get());
-        if (FAILED(hr))
-            goto exit;
-    }
-
-exit:
-    if (succeeded)
-        *succeeded = SUCCEEDED(hr);
-    return hr;
+    ASSERT_NOT_REACHED();
+    return E_FAIL;
 }
 
-static CFDictionaryRef createHistoryListFromStream(CFReadStreamRef stream, CFPropertyListFormat format)
+HRESULT STDMETHODCALLTYPE WebHistory::unused2()
 {
-    return (CFDictionaryRef)CFPropertyListCreateFromStream(0, stream, 0, kCFPropertyListImmutable, &format, 0);
-}
-
-HRESULT WebHistory::loadHistoryGutsFromURL(CFURLRef url, CFMutableArrayRef discardedItems, IWebError** /*error*/) //FIXME
-{
-    CFPropertyListFormat format = kCFPropertyListBinaryFormat_v1_0 | kCFPropertyListXMLFormat_v1_0;
-    HRESULT hr = S_OK;
-    int numberOfItemsLoaded = 0;
-
-    RetainPtr<CFReadStreamRef> stream(AdoptCF, CFReadStreamCreateWithFile(0, url));
-    if (!stream) 
-        return E_FAIL;
-
-    if (!CFReadStreamOpen(stream.get())) 
-        return E_FAIL;
-
-    RetainPtr<CFDictionaryRef> historyList(AdoptCF, createHistoryListFromStream(stream.get(), format));
-    CFReadStreamClose(stream.get());
-
-    if (!historyList) 
-        return E_FAIL;
-
-    CFNumberRef fileVersionObject = (CFNumberRef)CFDictionaryGetValue(historyList.get(), FileVersionKey);
-    int fileVersion;
-    if (!CFNumberGetValue(fileVersionObject, kCFNumberIntType, &fileVersion)) 
-        return E_FAIL;
-
-    if (fileVersion > currentFileVersion) 
-        return E_FAIL;
-    
-    CFArrayRef datesArray = (CFArrayRef)CFDictionaryGetValue(historyList.get(), DatesArrayKey);
-
-    int itemCountLimit;
-    hr = historyItemLimit(&itemCountLimit);
-    if (FAILED(hr))
-        return hr;
-
-    CFAbsoluteTime limitDate;
-    hr = ageLimitDate(&limitDate);
-    if (FAILED(hr))
-        return hr;
-
-    bool ageLimitPassed = false;
-    bool itemLimitPassed = false;
-
-    CFIndex itemCount = CFArrayGetCount(datesArray);
-    for (CFIndex i = 0; i < itemCount; ++i) {
-        CFDictionaryRef itemAsDictionary = (CFDictionaryRef)CFArrayGetValueAtIndex(datesArray, i);
-        COMPtr<WebHistoryItem> item(AdoptCOM, WebHistoryItem::createInstance());
-        hr = item->initFromDictionaryRepresentation((void*)itemAsDictionary);
-        if (FAILED(hr))
-            return hr;
-
-        // item without URL is useless; data on disk must have been bad; ignore
-        BOOL hasURL;
-        hr = item->hasURLString(&hasURL);
-        if (FAILED(hr))
-            return hr;
-        
-        if (hasURL) {
-            // Test against date limit. Since the items are ordered newest to oldest, we can stop comparing
-            // once we've found the first item that's too old.
-            if (!ageLimitPassed) {
-                DATE lastVisitedTime;
-                hr = item->lastVisitedTimeInterval(&lastVisitedTime);
-                if (FAILED(hr))
-                    return hr;
-                if (timeToDate(MarshallingHelpers::DATEToCFAbsoluteTime(lastVisitedTime)) <= limitDate)
-                    ageLimitPassed = true;
-            }
-            if (ageLimitPassed || itemLimitPassed)
-                CFArrayAppendValue(discardedItems, item.get());
-            else {
-                bool added;
-                addItem(item.get(), true, &added); // ref is added inside addItem
-                if (added)
-                    ++numberOfItemsLoaded;
-                if (numberOfItemsLoaded == itemCountLimit)
-                    itemLimitPassed = true;
-            }
-        }
-    }
-    return hr;
-}
-
-HRESULT STDMETHODCALLTYPE WebHistory::saveToURL( 
-    /* [in] */ BSTR url,
-    /* [out] */ IWebError** error,
-    /* [retval][out] */ BOOL* succeeded)
-{
-    HRESULT hr = S_OK;
-    RetainPtr<CFURLRef> urlRef(AdoptCF, MarshallingHelpers::BSTRToCFURLRef(url));
-
-    hr = saveHistoryGuts(urlRef.get(), error);
-
-    if (succeeded)
-        *succeeded = SUCCEEDED(hr);
-    if (SUCCEEDED(hr))
-        hr = postNotification(kWebHistorySavedNotification);
-
-    return hr;
-}
-
-HRESULT WebHistory::saveHistoryGuts(CFURLRef url, IWebError** error)
-{
-    HRESULT hr = S_OK;
-
-    // FIXME: Correctly report error when new API is ready.
-    if (error)
-        *error = 0;
-
-    RetainPtr<CFDataRef> data = this->data();
-
-    RetainPtr<CFWriteStreamRef> stream(AdoptCF, CFWriteStreamCreateWithFile(kCFAllocatorDefault, url));
-    if (!stream) 
-        return E_FAIL;
-
-    if (!CFWriteStreamOpen(stream.get())) 
-        return E_FAIL;
-
-    const UInt8* dataPtr = CFDataGetBytePtr(data.get());
-    CFIndex length = CFDataGetLength(data.get());
-
-    while (length) {
-        CFIndex bytesWritten = CFWriteStreamWrite(stream.get(), dataPtr, length);
-        if (bytesWritten <= 0) {
-            hr = E_FAIL;
-            break;
-        }
-        dataPtr += bytesWritten;
-        length -= bytesWritten;
-    }
-
-    CFWriteStreamClose(stream.get());
-
-    return hr;
+    ASSERT_NOT_REACHED();
+    return E_FAIL;
 }
 
 HRESULT STDMETHODCALLTYPE WebHistory::addItems( 
@@ -448,7 +247,7 @@ HRESULT STDMETHODCALLTYPE WebHistory::removeAllItems( void)
     CFIndex itemCount = CFDictionaryGetCount(m_entriesByURL.get());
     Vector<IWebHistoryItem*> itemsVector(itemCount);
     CFDictionaryGetKeysAndValues(m_entriesByURL.get(), 0, (const void**)itemsVector.data());
-    RetainPtr<CFArrayRef> allItems(AdoptCF, CFArrayCreate(kCFAllocatorDefault, (const void**)itemsVector.data(), itemCount, &MarshallingHelpers::kIUnknownArrayCallBacks));
+    RetainPtr<CFArrayRef> allItems = adoptCF(CFArrayCreate(kCFAllocatorDefault, (const void**)itemsVector.data(), itemCount, &MarshallingHelpers::kIUnknownArrayCallBacks));
 
     CFDictionaryRemoveAllValues(m_entriesByURL.get());
 
@@ -551,21 +350,6 @@ HRESULT STDMETHODCALLTYPE WebHistory::allItems(
     return S_OK;
 }
 
-HRESULT WebHistory::data(IStream** stream)
-{
-    if (!stream)
-        return E_POINTER;
-
-    *stream = 0;
-
-    RetainPtr<CFDataRef> historyData = data();
-    if (!historyData)
-        return S_OK;
-
-    COMPtr<MemoryStream> result = MemoryStream::createInstance(SharedBuffer::wrapCFData(historyData.get()));
-    return result.copyRefTo(stream);
-}
-
 HRESULT WebHistory::setVisitedLinkTrackingEnabled(BOOL visitedLinkTrackingEnabled)
 {
     PageGroup::setShouldTrackVisitedLinks(visitedLinkTrackingEnabled);
@@ -619,7 +403,7 @@ HRESULT WebHistory::removeItem(IWebHistoryItem* entry)
     if (FAILED(hr))
         return hr;
 
-    RetainPtr<CFStringRef> urlString(AdoptCF, MarshallingHelpers::BSTRToCFStringRef(urlBStr));
+    RetainPtr<CFStringRef> urlString = adoptCF(MarshallingHelpers::BSTRToCFStringRef(urlBStr));
 
     // If this exact object isn't stored, then make no change.
     // FIXME: Is this the right behavior if this entry isn't present, but another entry for the same URL is?
@@ -651,7 +435,7 @@ HRESULT WebHistory::addItem(IWebHistoryItem* entry, bool discardDuplicate, bool*
     if (FAILED(hr))
         return hr;
 
-    RetainPtr<CFStringRef> urlString(AdoptCF, MarshallingHelpers::BSTRToCFStringRef(urlBStr));
+    RetainPtr<CFStringRef> urlString = adoptCF(MarshallingHelpers::BSTRToCFStringRef(urlBStr));
 
     COMPtr<IWebHistoryItem> oldEntry((IWebHistoryItem*) CFDictionaryGetValue(
         m_entriesByURL.get(), urlString.get()));
@@ -765,7 +549,7 @@ HRESULT STDMETHODCALLTYPE WebHistory::itemForURL(
     /* [in] */ BSTR url,
     /* [retval][out] */ IWebHistoryItem** item)
 {
-    RetainPtr<CFStringRef> urlString(AdoptCF, MarshallingHelpers::BSTRToCFStringRef(url));
+    RetainPtr<CFStringRef> urlString = adoptCF(MarshallingHelpers::BSTRToCFStringRef(url));
     return itemForURLString(urlString.get(), item);
 }
 
@@ -808,7 +592,7 @@ HRESULT WebHistory::addItemToDateCaches(IWebHistoryItem* entry)
     } else {
         ASSERT(!m_entriesByDate.contains(dateKey));
         // no other entries exist for this date
-        RetainPtr<CFMutableArrayRef> entryArray(AdoptCF, 
+        RetainPtr<CFMutableArrayRef> entryArray = adoptCF(
             CFArrayCreateMutable(0, 0, &MarshallingHelpers::kIUnknownArrayCallBacks));
         CFArrayAppendValue(entryArray.get(), entry);
         m_entriesByDate.set(dateKey, entryArray);
@@ -852,7 +636,7 @@ HRESULT WebHistory::removeItemFromDateCaches(IWebHistoryItem* entry)
 
 static void getDayBoundaries(CFAbsoluteTime day, CFAbsoluteTime& beginningOfDay, CFAbsoluteTime& beginningOfNextDay)
 {
-    RetainPtr<CFTimeZoneRef> timeZone(AdoptCF, CFTimeZoneCopyDefault());
+    RetainPtr<CFTimeZoneRef> timeZone = adoptCF(CFTimeZoneCopyDefault());
     CFGregorianDate date = CFAbsoluteTimeGetGregorianDate(day, timeZone.get());
     date.hour = 0;
     date.minute = 0;
@@ -1000,13 +784,3 @@ void WebHistory::addVisitedLinksToPageGroup(PageGroup& group)
 {
     CFDictionaryApplyFunction(m_entriesByURL.get(), addVisitedLinkToPageGroup, &group);
 }
-
-RetainPtr<CFDataRef> WebHistory::data() const
-{
-    if (m_entriesByDate.isEmpty())
-        return 0;
-
-    WebHistoryWriter writer(m_entriesByDate);
-    writer.writePropertyList();
-    return writer.releaseData();
-}
